@@ -110,10 +110,6 @@
 
 #define INSIDE_TOLERANCE 1.0e-6
 
-/* Tolerance used for order reduction during root finding. */
-
-#define ROOT_TOLERANCE 1.0e-3
-
 /* Ray enters/exits a component. */
 
 #define ENTERING 0
@@ -125,13 +121,6 @@
 * Local typedefs
 ******************************************************************************/
 
-typedef struct
-{
-  int type;
-  DBL bound;
-  BLOB_ELEMENT *Element;
-} Blob_Interval;
-
 
 
 /*****************************************************************************
@@ -140,8 +129,8 @@ typedef struct
 
 static void element_normal PARAMS((VECTOR Result, VECTOR P, BLOB_ELEMENT *Element));
 static int intersect_element PARAMS((VECTOR P, VECTOR D, BLOB_ELEMENT *Element, DBL mindist, DBL *t0, DBL *t1));
-static void insert_hit PARAMS((BLOB_ELEMENT *Element, DBL t0, DBL t1, Blob_Interval *intervals, int *cnt));
-static int determine_influences PARAMS((VECTOR P, VECTOR D, BLOB *Blob, DBL mindist, Blob_Interval *intervals));
+static void insert_hit PARAMS((BLOB_ELEMENT *Element, DBL t0, DBL t1, BLOB_INTERVAL *intervals, int *cnt));
+static int determine_influences PARAMS((VECTOR P, VECTOR D, BLOB *Blob, DBL mindist, BLOB_INTERVAL *intervals));
 static DBL calculate_field_value PARAMS((BLOB *Blob, VECTOR P));
 static DBL calculate_element_field PARAMS((BLOB_ELEMENT *Element, VECTOR P));
 
@@ -302,11 +291,11 @@ ISTACK *Depth_Stack;
   DBL start_dist;
   VECTOR P, D, V1, PP, DD;
   VECTOR IPoint;
-  Blob_Interval *intervals;
+  BLOB_INTERVAL *intervals;
   BLOB_ELEMENT *Element;
   BLOB *Blob = (BLOB *)Object;
 
-  DBL l, m, w, newcoeffs[5], dk[5];
+  DBL l, w, newcoeffs[5], dk[5];
 
   Increase_Counter(stats[Ray_Blob_Tests]);
 
@@ -328,18 +317,14 @@ ISTACK *Depth_Stack;
     len = 1.0;
   }
 
-  /* Allocate temporary intersection list. */
-
-  intervals = (Blob_Interval *)POV_MALLOC(2*Blob->Data->Number_Of_Components*sizeof(Blob_Interval), "temporary blob intervals");
-
   /* Get the intervals along the ray where each component has an effect. */
-  
+
+  intervals = Blob->Data->Intervals;
+
   if ((cnt = determine_influences(P, D, Blob, DEPTH_TOLERANCE, intervals)) == 0)
   {
     /* Ray doesn't hit any of the component elements. */
-    
-    POV_FREE(intervals);
-    
+
     return (FALSE);
   }
 
@@ -384,7 +369,7 @@ ISTACK *Depth_Stack;
        */
       
       in_flag++;
-      
+
       Element = intervals[i].Element;
       
       switch (Element->Type)
@@ -432,7 +417,7 @@ ISTACK *Depth_Stack;
           fcoeffs[2] = 2.0 * c0 * (2.0 * t1 * t1 + t0 * t2) + c1 * t2;
           fcoeffs[3] = 2.0 * t1 * (2.0 * c0 * t0 + c1);
           fcoeffs[4] = t0 * (c0 * t0 + c1) + c2;
-        
+
           break;
         
         case BLOB_BASE_HEMISPHERE:
@@ -445,7 +430,7 @@ ISTACK *Depth_Stack;
           {
             PP[Z] -= Element->len;
           }
-        
+
           VDot(t0, PP, PP);
           VDot(t1, PP, DD);
           VDot(t2, DD, DD);
@@ -467,14 +452,14 @@ ISTACK *Depth_Stack;
         case BLOB_CYLINDER:
 
           /* Transform ray into cylinder space. */
-        
+
           MInvTransPoint(PP, P, Element->Trans);
           MInvTransDirection(DD, D, Element->Trans);
         
           t0 = PP[X] * PP[X] + PP[Y] * PP[Y];
           t1 = PP[X] * DD[X] + PP[Y] * DD[Y];
           t2 = DD[X] * DD[X] + DD[Y] * DD[Y];
-        
+
           c0 = Element->c[0];
           c1 = Element->c[1];
           c2 = Element->c[2];
@@ -490,7 +475,7 @@ ISTACK *Depth_Stack;
           break;
         
         default:
-        
+
           Error("Unknown blob component in All_Blob_Intersections().\n");
       }
       
@@ -507,7 +492,7 @@ ISTACK *Depth_Stack;
        */
       
       fcoeffs = intervals[i].Element->f;
-      
+
       for (j = 0; j < 5; j++)
       {
         coeffs[j] -= fcoeffs[j];
@@ -520,8 +505,8 @@ ISTACK *Depth_Stack;
         continue;
       }
     }
-    
-    /* 
+
+    /*
      * If the following intersection lies close to the current intersection
      * then first add/subtract next region before testing. [DB 7/94] 
      */
@@ -535,24 +520,22 @@ ISTACK *Depth_Stack;
      * Transform polynomial in a way that the interval boundaries are moved
      * to 0 and 1, i. e. the roots of interest are between 0 and 1. [DB 10/94]
      */
-    
-    l = intervals[i].bound;
-    m = intervals[i+1].bound;
 
-    w = m - l;
+    l = intervals[i].bound;
+    w = intervals[i+1].bound - l;
 
     newcoeffs[0] = coeffs[0] * w * w * w * w;
     newcoeffs[1] = (coeffs[1] + 4.0 * coeffs[0] * l) * w * w * w;
     newcoeffs[2] = (3.0 * l * (2.0 * coeffs[0] * l + coeffs[1]) + coeffs[2]) * w * w;
-    newcoeffs[3] = (2.0 * l * (l * (2.0 * coeffs[0] * l + 1.5 * coeffs[1]) + coeffs[2]) + coeffs[3]) * w;
+    newcoeffs[3] = (2.0 * l * (2.0 * l * (coeffs[0] * l + 0.75 * coeffs[1]) + coeffs[2]) + coeffs[3]) * w;
     newcoeffs[4] = l * (l * (l * (coeffs[0] * l + coeffs[1]) + coeffs[2]) + coeffs[3]) + coeffs[4];
 
     /* Calculate coefficients of corresponding bezier curve. [DB 10/94] */
 
     dk[0] = newcoeffs[4];
     dk[1] = newcoeffs[4] + 0.25 * newcoeffs[3];
-    dk[2] = newcoeffs[4] + 0.50 * newcoeffs[3] + newcoeffs[2] / 6.0;
-    dk[3] = newcoeffs[4] + 0.75 * newcoeffs[3] + 0.5 * newcoeffs[2] + 0.25 * newcoeffs[1];
+    dk[2] = newcoeffs[4] + 0.50 * (newcoeffs[3] + newcoeffs[2] / 12.0);
+    dk[3] = newcoeffs[4] + 0.50 * (0.375 * newcoeffs[3] + newcoeffs[2] + 0.125 * newcoeffs[1]);
     dk[4] = newcoeffs[4] + newcoeffs[3] + newcoeffs[2] + newcoeffs[1] + newcoeffs[0];
 
     /*
@@ -574,7 +557,7 @@ ISTACK *Depth_Stack;
 
     /* Solve polynomial. */
 
-    root_count = Solve_Polynomial(4, coeffs, roots, Test_Flag(Blob, STURM_FLAG), ROOT_TOLERANCE);
+    root_count = Solve_Polynomial(4, coeffs, roots, Test_Flag(Blob, STURM_FLAG), 0.0);
 
     /* See if any of the roots are valid. */
 
@@ -587,22 +570,23 @@ ISTACK *Depth_Stack;
        * the currently active components.
        */
 
-      if ((dist >= intervals[i].bound) && (dist <= intervals[i+1].bound) &&
-          (dist > DEPTH_TOLERANCE) && (dist < Max_Distance))
+      if ((dist >= intervals[i].bound) &&
+          (dist <= intervals[i+1].bound))
       {
         /* Correct distance. */
 
-        dist += start_dist;
+        dist = (dist + start_dist) / len;
 
-        dist /= len;
-
-        VEvaluateRay(IPoint, Ray->Initial, dist, Ray->Direction);
-
-        if (Point_In_Clip(IPoint, Object->Clip))
+        if ((dist > DEPTH_TOLERANCE) && (dist < Max_Distance))
         {
-          push_entry(dist, IPoint, Object, Depth_Stack);
+          VEvaluateRay(IPoint, Ray->Initial, dist, Ray->Direction);
 
-          Intersection_Found = TRUE;
+          if (Point_In_Clip(IPoint, Object->Clip))
+          {
+            push_entry(dist, IPoint, Object, Depth_Stack);
+
+            Intersection_Found = TRUE;
+          }
         }
       }
     }
@@ -624,10 +608,6 @@ ISTACK *Depth_Stack;
     Increase_Counter(stats[Ray_Blob_Tests_Succeeded]);
   }
 
-  /* Get rid off temporary intersection list. */
-
-  POV_FREE(intervals);
-
   return (Intersection_Found);
 }
 
@@ -658,31 +638,34 @@ ISTACK *Depth_Stack;
 *
 * DESCRIPTION
 *
-*   -
+*   Store the points of intersection. Keep track of: whether this is
+*   the start or end point of the hit, which component was pierced
+*   by the ray, and the point along the ray that the hit occured at.
 *
 * CHANGES
 *
 *   Oct 1994 : Modified to use memmove instead of loops for copying. [DB]
 *   Sep 1995 : Changed to allow use of memcpy if memmove isn't available. [AED]
 *   Jul 1996 : Changed to use POV_MEMMOVE, which can be memmove or pov_memmove.
+*   Oct 1996 : Changed to avoid unnecessary compares. [DB]
 *
 ******************************************************************************/
 
 static void insert_hit(Element, t0, t1, intervals, cnt)
 BLOB_ELEMENT *Element;
 DBL t0, t1;
-Blob_Interval *intervals;
+BLOB_INTERVAL *intervals;
 int *cnt;
 {
   int k;
 
-  /*
-   * Store the points of intersection. Keep track of: whether this is
-   * the start or end point of the hit, which component was pierced
-   * by the ray, and the point along the ray that the hit occured at.
-   */
+  /* We are entering the component. */
 
-  for (k = 0; (k < *cnt) && (t0 > intervals[k].bound); k++);
+  intervals[*cnt].type    = Element->Type | ENTERING;
+  intervals[*cnt].bound   = t0;
+  intervals[*cnt].Element = Element;
+
+  for (k = 0; t0 > intervals[k].bound; k++);
 
   if (k < *cnt)
   {
@@ -691,7 +674,7 @@ int *cnt;
      * bump the rest and insert it here.
      */
 
-    POV_MEMMOVE(&intervals[k+1], &intervals[k], (*cnt-k)*sizeof(Blob_Interval));
+    POV_MEMMOVE(&intervals[k+1], &intervals[k], (*cnt-k)*sizeof(BLOB_INTERVAL));
 
     /* We are entering the component. */
 
@@ -701,11 +684,17 @@ int *cnt;
 
     (*cnt)++;
 
-    for (k = k + 1; (k < *cnt) && (t1 > intervals[k].bound); k++);
+    /* We are exiting the component. */
+
+    intervals[*cnt].type    = Element->Type | EXITING;
+    intervals[*cnt].bound   = t1;
+    intervals[*cnt].Element = Element;
+
+    for (k = k + 1; t1 > intervals[k].bound; k++);
 
     if (k < *cnt)
     {
-      POV_MEMMOVE(&intervals[k+1], &intervals[k], (*cnt-k)*sizeof(Blob_Interval));
+      POV_MEMMOVE(&intervals[k+1], &intervals[k], (*cnt-k)*sizeof(BLOB_INTERVAL));
 
       /* We are exiting the component. */
 
@@ -713,14 +702,6 @@ int *cnt;
       intervals[k].bound   = t1;
       intervals[k].Element = Element;
     }
-    else
-    {
-      /* We are exiting the component. */
-
-      intervals[*cnt].type    = Element->Type | EXITING;
-      intervals[*cnt].bound   = t1;
-      intervals[*cnt].Element = Element;
-    }
 
     (*cnt)++;
   }
@@ -728,12 +709,6 @@ int *cnt;
   {
     /* Just plop the start and end points at the end of the list */
 
-    /* We are entering the component. */
-
-    intervals[*cnt].type    = Element->Type | ENTERING;
-    intervals[*cnt].bound   = t0;
-    intervals[*cnt].Element = Element;
-
     (*cnt)++;
 
     /* We are exiting the component. */
@@ -1352,7 +1327,7 @@ static int determine_influences(P, D, Blob, mindist, intervals)
 VECTOR P, D;
 BLOB *Blob;
 DBL mindist;
-Blob_Interval *intervals;
+BLOB_INTERVAL *intervals;
 {
   int i;
   int cnt, size;
@@ -2326,7 +2301,7 @@ OBJECT *Object;
 *
 * RETURNS
 *
-*   blobstackptr - Pointer to blob element
+*   BLOB_LIST * - Pointer to blob element
 *
 * AUTHOR
 *
@@ -2342,11 +2317,11 @@ OBJECT *Object;
 *
 ******************************************************************************/
 
-blobstackptr Create_Blob_List_Element()
+BLOB_LIST *Create_Blob_List_Element()
 {
-  blobstackptr New;
+  BLOB_LIST *New;
 
-  New = (blobstackptr)POV_MALLOC(sizeof(struct blob_list_struct), "blob component");
+  New = (BLOB_LIST *)POV_MALLOC(sizeof(BLOB_LIST), "blob component");
 
   init_blob_element(&New->elem);
 
@@ -2432,6 +2407,8 @@ OBJECT *Object;
 
     POV_FREE(Blob->Data->Entry);
 
+    POV_FREE(Blob->Data->Intervals);
+
     POV_FREE(Blob->Data);
   }
 
@@ -2708,12 +2685,12 @@ BLOB_ELEMENT *Element;
 void Make_Blob(Blob, threshold, BlobList, npoints)
 BLOB *Blob;
 DBL threshold;
-blobstackptr BlobList;
+BLOB_LIST *BlobList;
 int npoints;
 {
   int i, count;
   DBL rad2, coeff;
-  blobstackptr temp;
+  BLOB_LIST *temp;
   BLOB_ELEMENT *Entry;
 
   if (npoints < 1)
@@ -3623,11 +3600,11 @@ int *size;
 
 void Create_Blob_Element_Texture_List(Blob, BlobList, npoints)
 BLOB *Blob;
-blobstackptr BlobList;
+BLOB_LIST *BlobList;
 int npoints;
 {
   int i, element_count, count;
-  blobstackptr temp;
+  BLOB_LIST *temp;
 
   if (npoints < 1)
   {
@@ -3665,8 +3642,12 @@ int npoints;
 
   Blob->Data->References = 1;
 
+  Blob->Data->Number_Of_Components = count;
+
   Blob->Data->Entry = (BLOB_ELEMENT *)POV_MALLOC(count*sizeof(BLOB_ELEMENT), "blob data");
 
+  Blob->Data->Intervals = (BLOB_INTERVAL *)POV_MALLOC(2*Blob->Data->Number_Of_Components*sizeof(BLOB_INTERVAL), "blob intervals");
+
   Blob->Data->Tree = NULL;
 
   /* Init components. */
@@ -3676,8 +3657,6 @@ int npoints;
     init_blob_element(&Blob->Data->Entry[i]);
   }
 
-  Blob->Data->Number_Of_Components = count;
-
   /* Allocate memory for list. */
 
   Blob->Element_Texture = (TEXTURE **)POV_MALLOC(count*sizeof(TEXTURE *), "blob texture list");
