@@ -1971,6 +1971,10 @@ BLEND_MAP *Parse_Colour_Map ()
                j++;
                EXIT
              END_CASE
+             
+             OTHERWISE
+               Parse_Error_Str ("float or color");
+             END_CASE
 
            END_EXPECT
 
@@ -2105,10 +2109,7 @@ char *Parse_String()
           Parse_Comma();
           temp1=New;
           temp2=Parse_String();
-          if ((l2=strlen(temp1)+strlen(temp2)+2) > 400)
-          {
-             Error("String too long.");
-          }
+          l2=strlen(temp1)+strlen(temp2)+2;
           New=POV_MALLOC(l2, "temporary string");
           strcpy(New,temp1);
           strcat(New,temp2);
@@ -2227,8 +2228,7 @@ char *Parse_Formatted_String()
 
         case 'v': *dest=0x0b; break;
 
-        case '\0': 
-        case '\\': *dest=0x5c; break;
+        case '\0': *dest=0x5c; break;
 
         case '\'': *dest=0x27; break;
            
