@@ -127,13 +127,9 @@
 
 #define DEPTH_TOLERANCE 1.0e-4
 
-/* |Coefficients| < COEFF_LIMIT are regarded to be 0. */
-
-#define COEFF_LIMIT 1.0e-20
-
 /* Max. number of intersecions per spline segment. */
 
-#define MAX_INTERSECTIONS_PER_SEGMENT 6
+#define MAX_INTERSECTIONS_PER_SEGMENT 4
 
 
 
@@ -141,32 +137,25 @@
 * Local typedefs
 ******************************************************************************/
 
-typedef struct Lathe_Intersection_Structure LATHE_INT;
-
-struct Lathe_Intersection_Structure
-{
-  DBL d;  /* Distance of intersection point                  */
-  DBL w;  /* Paramter of intersection point with n-th spline */
-  int n;  /* Number of segment hit                           */
-};
-
 
 
 /*****************************************************************************
 * Static functions
 ******************************************************************************/
 
-static int intersect_lathe PARAMS((RAY *Ray, LATHE *Lathe, LATHE_INT *Int));
-static int   All_Lathe_Intersections PARAMS((OBJECT *Object, RAY *Ray, ISTACK *Depth_Stack));
-static int   Inside_Lathe PARAMS((VECTOR point, OBJECT *Object));
-static void  Lathe_Normal PARAMS((VECTOR Result, OBJECT *Object, INTERSECTION *Inter));
-static void  *Copy_Lathe PARAMS((OBJECT *Object));
-static void  Translate_Lathe PARAMS((OBJECT *Object, VECTOR Vector, TRANSFORM *Trans));
-static void  Rotate_Lathe PARAMS((OBJECT *Object, VECTOR Vector, TRANSFORM *Trans));
-static void  Scale_Lathe PARAMS((OBJECT *Object, VECTOR Vector, TRANSFORM *Trans));
-static void  Transform_Lathe PARAMS((OBJECT *Object, TRANSFORM *Trans));
-static void  Invert_Lathe PARAMS((OBJECT *Object));
-static void  Destroy_Lathe PARAMS((OBJECT *Object));
+static int  intersect_lathe PARAMS((RAY *Ray, LATHE *Lathe, ISTACK *Depth_Stack));
+static int  All_Lathe_Intersections PARAMS((OBJECT *Object, RAY *Ray, ISTACK *Depth_Stack));
+static int  Inside_Lathe PARAMS((VECTOR point, OBJECT *Object));
+static void Lathe_Normal PARAMS((VECTOR Result, OBJECT *Object, INTERSECTION *Inter));
+static void *Copy_Lathe PARAMS((OBJECT *Object));
+static void Translate_Lathe PARAMS((OBJECT *Object, VECTOR Vector, TRANSFORM *Trans));
+static void Rotate_Lathe PARAMS((OBJECT *Object, VECTOR Vector, TRANSFORM *Trans));
+static void Scale_Lathe PARAMS((OBJECT *Object, VECTOR Vector, TRANSFORM *Trans));
+static void Transform_Lathe PARAMS((OBJECT *Object, TRANSFORM *Trans));
+static void Invert_Lathe PARAMS((OBJECT *Object));
+static void Destroy_Lathe PARAMS((OBJECT *Object));
+
+static int  test_hit PARAMS((LATHE *, RAY *, ISTACK *, DBL, DBL, int));
 
 
 /*****************************************************************************
@@ -195,11 +184,11 @@ static METHODS Lathe_Methods =
 *   Object      - Object
 *   Ray         - Ray
 *   Depth_Stack - Intersection stack
-*   
+*
 * OUTPUT
 *
 *   Depth_Stack
-*   
+*
 * RETURNS
 *
 *   int - TRUE, if a intersection was found
@@ -215,6 +204,7 @@ static METHODS Lathe_Methods =
 * CHANGES
 *
 *   Jun 1994 : Creation.
+*   Oct 1996 : Changed code to include faster version. [DB]
 *
 ******************************************************************************/
 
@@ -223,39 +213,16 @@ OBJECT *Object;
 RAY *Ray;
 ISTACK *Depth_Stack;
 {
-  int i, imax, Found;
-  LATHE_INT *I;
-  VECTOR IPoint;
+  Increase_Counter(stats[Ray_Lathe_Tests]);
 
-  /* Allocate memory for intersections (two per segment). */
-
-  I = (LATHE_INT *)POV_MALLOC(MAX_INTERSECTIONS_PER_SEGMENT*((LATHE *)Object)->Number*sizeof(LATHE_INT), "lathe intersection array");
-
-  Found = FALSE;
-
-  imax = intersect_lathe(Ray, (LATHE *)Object, I);
-
-  if (imax)
+  if (intersect_lathe(Ray, (LATHE *)Object, Depth_Stack))
   {
-    for (i = 0; i < imax; i++)
-    {
-      if ((I[i].d > DEPTH_TOLERANCE) && (I[i].d < Max_Distance))
-      {
-        VEvaluateRay(IPoint, Ray->Initial, I[i].d, Ray->Direction);
+    Increase_Counter(stats[Ray_Lathe_Tests_Succeeded]);
 
-        if (Point_In_Clip(IPoint, Object->Clip))
-        {
-          push_entry_i1_d1(I[i].d,IPoint,Object,I[i].n,I[i].w,Depth_Stack);
-
-          Found = TRUE;
-        }
-      }
-    }
+    return(TRUE);
   }
 
-  POV_FREE (I);
-
-  return(Found);
+  return(FALSE);
 }
 
 
@@ -271,50 +238,46 @@ ISTACK *Depth_Stack;
 *   Ray          - Ray
 *   Lathe        - Lathe
 *   Intersection - Lathe intersection structure
-*   
+*
 * OUTPUT
 *
 *   Intersection
-*   
+*
 * RETURNS
 *
 *   int - Number of intersections found
-*   
+*
 * AUTHOR
 *
 *   Dieter Bayer
-*   
+*
 * DESCRIPTION
 *
 *   Determine ray/lathe intersection.
 *
 *   NOTE: The curve is rotated about the y-axis!
-*         Order reduction cannot be used.
+*         Order of the polynomial must not be used!
 *
 * CHANGES
 *
 *   Jun 1994 : Creation.
+*   Oct 1996 : Changed code to include faster version. [DB]
 *
 ******************************************************************************/
 
-static int intersect_lathe(Ray, Lathe, Intersection)
+static int intersect_lathe(Ray, Lathe, Depth_Stack)
 RAY *Ray;
 LATHE *Lathe;
-LATHE_INT *Intersection;
+ISTACK *Depth_Stack;
 {
-  int i, ii, j, n1, n2, skip;
+  int cnt;
+  int found, j, n1, n2;
   DBL k, len, r, m, w, Dy2, r0;
   DBL x1[7], x2[3], y1[6], y2[2];
+  DBL best;
   VECTOR P, D;
-  LATHE_SPLINE_ENTRY Entry;
-
-  Increase_Counter(stats[Ray_Lathe_Tests]);
-
-  /* Init intersection structure. */
-
-  Intersection->d = 
-  Intersection->w = 0.0;
-  Intersection->n = 0;
+  BCYL_INT *intervals;
+  LATHE_SPLINE_ENTRY *Entry;
 
   /* Transform the ray into the lathe space. */
 
@@ -340,7 +303,7 @@ LATHE_INT *Intersection;
     return(FALSE);
   }
 
-  /* Get distance r0 of the ray from rotation axis (= y axis). */
+  /* Get distance r0 of the ray from rotation axis (i.e. y axis). */
 
   r0 = fabs(P[X] * D[Z] - P[Z] * D[X]);
 
@@ -358,49 +321,50 @@ LATHE_INT *Intersection;
     return(FALSE);
   }
 
+  /* Intersect all cylindrical bounds. */
+
+  if ((cnt = Intersect_BCyl(Lathe->Spline->BCyl, P, D)) == 0)
+  {
+    return(FALSE);
+  }
+
 #ifdef LATHE_EXTRA_STATS
   Increase_Counter(stats[Lathe_Bound_Tests_Succeeded]);
 #endif
 
-  /* Init number of intersections found. */
-
-  i = 0;
-
   /* Precalculate some constants that are ray-dependant only. */
 
   m = D[X] * P[X] + D[Z] * P[Z];
 
   Dy2 = D[Y] * D[Y];
 
-  /* Intersect all spline segments. */
+  /* Step through the list of intersections. */
 
-  for (j = 0; j < Lathe->Number; j++)
+  found = FALSE;
+
+  best = BOUND_HUGE;
+
+  intervals = Lathe->Spline->BCyl->intervals;
+
+  for (j = 0; j < cnt; j++)
   {
-    Entry = Lathe->Spline->Entry[j];
+    /* Get current segment. */
 
-    /* Test if ray misses segment's cylindrical bound. */
+    Entry = &Lathe->Spline->Entry[intervals[j].n];
 
-#ifdef LATHE_EXTRA_STATS
-    Increase_Counter(stats[Lathe_Bound_Tests]);
-#endif
+    /* If we already have the best intersection we may exit. */
 
-    if (r0 > Entry.r2)
+    if (!(Lathe->Type & IS_CHILD_OBJECT) && (intervals[j].d[0] > best))
     {
-      continue;
+      break;
     }
 
-    if (((D[Y] >= 0.0) && (P[Y] >  Entry.h2)) ||
-        ((D[Y] <= 0.0) && (P[Y] <  Entry.h1)) ||
-        ((D[X] >= 0.0) && (P[X] >  Entry.r2)) ||
-        ((D[X] <= 0.0) && (P[X] < -Entry.r2)))
-    {
-      continue;
-    }
-
-    /* Number of roots found. */
+    /* Init number of roots found. */
 
     n1 = 0;
 
+    /* Intersect segment. */
+
     switch (Lathe->Spline_Type)
     {
       /***********************************************************************
@@ -409,17 +373,13 @@ LATHE_INT *Intersection;
 
       case LINEAR_SPLINE:
 
-#ifdef LATHE_EXTRA_STATS
-        Increase_Counter(stats[Lathe_Bound_Tests_Succeeded]);
-#endif
-
         /* Solve 2th order polynomial. */
 
-        x1[0] = Entry.C[Y] * Entry.C[Y] * r - Entry.C[X] * Entry.C[X] * Dy2;
+        x1[0] = Entry->C[Y] * Entry->C[Y] * r - Entry->C[X] * Entry->C[X] * Dy2;
 
-        x1[1] = 2.0 * (Entry.C[Y] * ((Entry.D[Y] - P[Y]) * r + D[Y] * m) - Entry.C[X] * Entry.D[X] * Dy2);
+        x1[1] = 2.0 * (Entry->C[Y] * ((Entry->D[Y] - P[Y]) * r + D[Y] * m) - Entry->C[X] * Entry->D[X] * Dy2);
 
-        x1[2] = (Entry.D[Y] - P[Y]) * ((Entry.D[Y] - P[Y]) * r + 2.0 * D[Y] * m) + Dy2 * (P[X] * P[X] + P[Z] * P[Z] - Entry.D[X] * Entry.D[X]);
+        x1[2] = (Entry->D[Y] - P[Y]) * ((Entry->D[Y] - P[Y]) * r + 2.0 * D[Y] * m) + Dy2 * (P[X] * P[X] + P[Z] * P[Z] - Entry->D[X] * Entry->D[X]);
 
         n1 = Solve_Polynomial(2, x1, y1, FALSE, 0.0);
 
@@ -431,28 +391,19 @@ LATHE_INT *Intersection;
 
       case QUADRATIC_SPLINE:
 
-        /* First test the bounding cylinder. */
+        /* Solve 4th order polynomial. */
 
-        if (Test_Thick_Cylinder(P, D, Entry.h1, Entry.h2, Sqr(Entry.r1), Sqr(Entry.r2)))
-        {
-#ifdef LATHE_EXTRA_STATS
-          Increase_Counter(stats[Lathe_Bound_Tests_Succeeded]);
-#endif
+        x1[0] = Entry->B[Y] * Entry->B[Y] * r - Entry->B[X] * Entry->B[X] * Dy2;
 
-          /* Solve 4th order polynomial. */
+        x1[1] = 2.0 * (Entry->B[Y] * Entry->C[Y] * r - Entry->B[X] * Entry->C[X] * Dy2);
 
-          x1[0] = Entry.B[Y] * Entry.B[Y] * r - Entry.B[X] * Entry.B[X] * Dy2;
+        x1[2] = r * (2.0 * Entry->B[Y] * (Entry->D[Y] - P[Y]) + Entry->C[Y] * Entry->C[Y]) + 2.0 * Entry->B[Y] * D[Y] * m - (2.0 * Entry->B[X] * Entry->D[X] + Entry->C[X] * Entry->C[X]) * Dy2;
 
-          x1[1] = 2.0 * (Entry.B[Y] * Entry.C[Y] * r - Entry.B[X] * Entry.C[X] * Dy2);
+        x1[3] = 2.0 * (Entry->C[Y] * ((Entry->D[Y] - P[Y]) * r + D[Y] * m) - Entry->C[X] * Entry->D[X] * Dy2);
 
-          x1[2] = r * (2.0 * Entry.B[Y] * (Entry.D[Y] - P[Y]) + Entry.C[Y] * Entry.C[Y]) + 2.0 * Entry.B[Y] * D[Y] * m - (2.0 * Entry.B[X] * Entry.D[X] + Entry.C[X] * Entry.C[X]) * Dy2;
+        x1[4] = (Entry->D[Y] - P[Y]) * ((Entry->D[Y] - P[Y]) * r + 2.0 * D[Y] * m) + Dy2 * (P[X] * P[X] + P[Z] * P[Z] - Entry->D[X] * Entry->D[X]);
 
-          x1[3] = 2.0 * (Entry.C[Y] * ((Entry.D[Y] - P[Y]) * r + D[Y] * m) - Entry.C[X] * Entry.D[X] * Dy2);
-
-          x1[4] = (Entry.D[Y] - P[Y]) * ((Entry.D[Y] - P[Y]) * r + 2.0 * D[Y] * m) + Dy2 * (P[X] * P[X] + P[Z] * P[Z] - Entry.D[X] * Entry.D[X]);
-
-          n1 = Solve_Polynomial(4, x1, y1, Test_Flag(Lathe, STURM_FLAG), 0.0);
-        }
+        n1 = Solve_Polynomial(4, x1, y1, Test_Flag(Lathe, STURM_FLAG), 0.0);
 
         break;
 
@@ -462,32 +413,23 @@ LATHE_INT *Intersection;
 
       case CUBIC_SPLINE:
 
-        /* First test the bounding cylinder. */
+        /* Solve 6th order polynomial. */
 
-        if (Test_Thick_Cylinder(P, D, Entry.h1, Entry.h2, Sqr(Entry.r1), Sqr(Entry.r2)))
-        {
-#ifdef LATHE_EXTRA_STATS
-          Increase_Counter(stats[Lathe_Bound_Tests_Succeeded]);
-#endif
+        x1[0] = Entry->A[Y] * Entry->A[Y] * r - Entry->A[X] * Entry->A[X] * Dy2;
 
-          /* Solve 6th order polynomial. */
+        x1[1] = 2.0 * (Entry->A[Y] * Entry->B[Y] * r - Entry->A[X] * Entry->B[X] * Dy2);
 
-          x1[0] = Entry.A[Y] * Entry.A[Y] * r - Entry.A[X] * Entry.A[X] * Dy2;
+        x1[2] = (2.0 * Entry->A[Y] * Entry->C[Y] + Entry->B[Y] * Entry->B[Y]) * r - (2.0 * Entry->A[X] * Entry->C[X] + Entry->B[X] * Entry->B[X]) * Dy2;
 
-          x1[1] = 2.0 * (Entry.A[Y] * Entry.B[Y] * r - Entry.A[X] * Entry.B[X] * Dy2);
+        x1[3] = 2.0 * ((Entry->A[Y] * Entry->D[Y] + Entry->B[Y] * Entry->C[Y] - Entry->A[Y] * P[Y]) * r + Entry->A[Y] * D[Y] * m - (Entry->A[X] * Entry->D[X] + Entry->B[X] * Entry->C[X]) * Dy2);
 
-          x1[2] = (2.0 * Entry.A[Y] * Entry.C[Y] + Entry.B[Y] * Entry.B[Y]) * r - (2.0 * Entry.A[X] * Entry.C[X] + Entry.B[X] * Entry.B[X]) * Dy2;
+        x1[4] = (2.0 * Entry->B[Y] * (Entry->D[Y] - P[Y]) + Entry->C[Y] * Entry->C[Y]) * r + 2.0 * Entry->B[Y] * D[Y] * m - (2.0 * Entry->B[X] * Entry->D[X] + Entry->C[X] * Entry->C[X]) * Dy2;
 
-          x1[3] = 2.0 * ((Entry.A[Y] * Entry.D[Y] + Entry.B[Y] * Entry.C[Y] - Entry.A[Y] * P[Y]) * r + Entry.A[Y] * D[Y] * m - (Entry.A[X] * Entry.D[X] + Entry.B[X] * Entry.C[X]) * Dy2);
+        x1[5] = 2.0 * (Entry->C[Y] * ((Entry->D[Y] - P[Y]) * r + D[Y] * m) - Entry->C[X] * Entry->D[X] * Dy2);
 
-          x1[4] = (2.0 * Entry.B[Y] * (Entry.D[Y] - P[Y]) + Entry.C[Y] * Entry.C[Y]) * r + 2.0 * Entry.B[Y] * D[Y] * m - (2.0 * Entry.B[X] * Entry.D[X] + Entry.C[X] * Entry.C[X]) * Dy2;
+        x1[6] = (Entry->D[Y] - P[Y]) * ((Entry->D[Y] - P[Y]) * r + 2.0 * D[Y] * m) + Dy2 * (P[X] * P[X] + P[Z] * P[Z] - Entry->D[X] * Entry->D[X]);
 
-          x1[5] = 2.0 * (Entry.C[Y] * ((Entry.D[Y] - P[Y]) * r + D[Y] * m) - Entry.C[X] * Entry.D[X] * Dy2);
-
-          x1[6] = (Entry.D[Y] - P[Y]) * ((Entry.D[Y] - P[Y]) * r + 2.0 * D[Y] * m) + Dy2 * (P[X] * P[X] + P[Z] * P[Z] - Entry.D[X] * Entry.D[X]);
-
-          n1 = Solve_Polynomial(6, x1, y1, Test_Flag(Lathe, STURM_FLAG), 0.0);
-        }
+        n1 = Solve_Polynomial(6, x1, y1, Test_Flag(Lathe, STURM_FLAG), 0.0);
 
         break;
     }
@@ -502,20 +444,21 @@ LATHE_INT *Intersection;
       {
         if (fabs(D[Y]) > EPSILON)
         {
-          k = (w * (w * (w * Entry.A[Y] + Entry.B[Y]) + Entry.C[Y]) + Entry.D[Y] - P[Y]) / D[Y];
+          k = (w * (w * (w * Entry->A[Y] + Entry->B[Y]) + Entry->C[Y]) + Entry->D[Y] - P[Y]) / D[Y];
 
-          Intersection[i].n   = j;
-          Intersection[i].w   = w;
-          Intersection[i++].d = k / len;
-
-          if (i > MAX_INTERSECTIONS_PER_SEGMENT * Lathe->Number)
+          if (test_hit(Lathe, Ray, Depth_Stack, k / len, w, intervals[j].n))
           {
-            Error("Too many intersections in intersect_lathe() found.\n");
+            found = TRUE;
+
+            if (k < best)
+            {
+              best = k;
+            }
           }
         }
         else
         {
-          k = w * (w * (w * Entry.A[X] + Entry.B[X]) + Entry.C[X]) + Entry.D[X];
+          k = w * (w * (w * Entry->A[X] + Entry->B[X]) + Entry->C[X]) + Entry->D[X];
 
           x2[0] = r;
           x2[1] = 2.0 * m;
@@ -526,31 +469,15 @@ LATHE_INT *Intersection;
 
           while (n2--)
           {
-            k = y2[n2] / len;
+            k = y2[n2];
 
-            /* Test if we already have this intersection. */
-
-            for (ii = skip = 0; ii < i; ii++)
+            if (test_hit(Lathe, Ray, Depth_Stack, k / len, w, intervals[j].n))
             {
-              if (fabs(Intersection[ii].d - k) < EPSILON)
+              found = TRUE;
+
+              if (k < best)
               {
-                skip = TRUE;
-
-                break;
-              }
-            }
-
-            /* Use it if we don't. */
-
-            if (!skip)
-            {
-              Intersection[i].n   = j;
-              Intersection[i].w   = w;
-              Intersection[i++].d = k;
-
-              if (i > MAX_INTERSECTIONS_PER_SEGMENT * Lathe->Number)
-              {
-                Error("Too many intersections in intersect_lathe() found.\n");
+                best = k;
               }
             }
           }
@@ -559,12 +486,7 @@ LATHE_INT *Intersection;
     }
   }
 
-  if (i)
-  {
-    Increase_Counter(stats[Ray_Lathe_Tests_Succeeded]);
-  }
-
-  return(i);
+  return(found);
 }
 
 
@@ -607,10 +529,18 @@ OBJECT *Object;
   int i, n, NC;
   DBL r, k, w;
   DBL x[4], y[3];
+  DBL *height, *radius;
   VECTOR P;
-  LATHE_SPLINE_ENTRY Entry;
+  BCYL_ENTRY *entry;
+  LATHE_SPLINE_ENTRY *Entry;
   LATHE *Lathe = (LATHE *)Object;
 
+  height = Lathe->Spline->BCyl->height;
+
+  radius = Lathe->Spline->BCyl->radius;
+
+  entry = Lathe->Spline->BCyl->entry;
+
   /* Transform the point into the lathe space. */
 
   MInvTransPoint(P, IPoint, Lathe->Trans);
@@ -627,17 +557,17 @@ OBJECT *Object;
     {
       for (i = 0; i < Lathe->Number; i++)
       {
-        Entry = Lathe->Spline->Entry[i];
+        Entry = &Lathe->Spline->Entry[i];
 
         /* Test if we are inside the segments cylindrical bound. */
 
-        if ((P[Y] - Entry.h1 >= -EPSILON) &&
-            (Entry.h2 - P[Y] >= -EPSILON) && (r <= Entry.r2))
+        if ((P[Y] >= height[entry[i].h1] - EPSILON) &&
+            (P[Y] <= height[entry[i].h2] + EPSILON))
         {
-          x[0] = Entry.A[Y];
-          x[1] = Entry.B[Y];
-          x[2] = Entry.C[Y];
-          x[3] = Entry.D[Y] - P[Y];
+          x[0] = Entry->A[Y];
+          x[1] = Entry->B[Y];
+          x[2] = Entry->C[Y];
+          x[3] = Entry->D[Y] - P[Y];
 
           n = Solve_Polynomial(3, x, y, Test_Flag(Lathe, STURM_FLAG), 0.0);
 
@@ -647,7 +577,7 @@ OBJECT *Object;
 
             if ((w >= 0.0) && (w <= 1.0))
             {
-              k  = w * (w * (w * Entry.A[X] + Entry.B[X]) + Entry.C[X]) + Entry.D[X] - r;
+              k  = w * (w * (w * Entry->A[X] + Entry->B[X]) + Entry->C[X]) + Entry->D[X] - r;
 
               if (k >= 0.0)
               {
@@ -712,9 +642,9 @@ INTERSECTION *Inter;
   DBL r, dx, dy;
   VECTOR P, N;
   LATHE *Lathe = (LATHE *)Object;
-  LATHE_SPLINE_ENTRY Entry;
+  LATHE_SPLINE_ENTRY *Entry;
 
-  Entry = Lathe->Spline->Entry[Inter->i1];
+  Entry = &Lathe->Spline->Entry[Inter->i1];
 
   /* Transform the point into the lathe space. */
 
@@ -730,8 +660,8 @@ INTERSECTION *Inter;
 
     /* Get derivatives. */
 
-    dx = Inter->d1 * (3.0 * Entry.A[X] * Inter->d1 + 2.0 * Entry.B[X]) + Entry.C[X];
-    dy = Inter->d1 * (3.0 * Entry.A[Y] * Inter->d1 + 2.0 * Entry.B[Y]) + Entry.C[Y];
+    dx = Inter->d1 * (3.0 * Entry->A[X] * Inter->d1 + 2.0 * Entry->B[X]) + Entry->C[X];
+    dy = Inter->d1 * (3.0 * Entry->A[Y] * Inter->d1 + 2.0 * Entry->B[Y]) + Entry->C[Y];
 
     /* Get normal by rotation. */
 
@@ -1084,11 +1014,11 @@ OBJECT *Object;
 *   Object
 *   
 * RETURNS
-*   
+*
 * AUTHOR
 *
 *   Dieter Bayer
-*   
+*
 * DESCRIPTION
 *
 *   Destroy a lathe.
@@ -1098,6 +1028,7 @@ OBJECT *Object;
 * CHANGES
 *
 *   Jun 1994 : Creation.
+*   Oct 1996 : Changed code to include faster version. [DB]
 *
 ******************************************************************************/
 
@@ -1110,8 +1041,11 @@ OBJECT *Object;
 
   if (--(Lathe->Spline->References) == 0)
   {
-    POV_FREE (Lathe->Spline->Entry);
-    POV_FREE (Lathe->Spline);
+    Destroy_BCyl(Lathe->Spline->BCyl);
+
+    POV_FREE(Lathe->Spline->Entry);
+
+    POV_FREE(Lathe->Spline);
   }
 
   POV_FREE (Object);
@@ -1176,11 +1110,11 @@ LATHE *Lathe;
 *   Lathe
 *   
 * RETURNS
-*   
+*
 * AUTHOR
 *
 *   Dieter Bayer
-*   
+*
 * DESCRIPTION
 *
 *   Calculate the spline segments of a lathe from a set of points.
@@ -1190,6 +1124,7 @@ LATHE *Lathe;
 * CHANGES
 *
 *   Jun 1994 : Creation.
+*   Oct 1996 : Changed code to include faster version. [DB]
 *
 ******************************************************************************/
 
@@ -1201,6 +1136,10 @@ UV_VECT *P;
   DBL x[4], xmin, xmax;
   DBL y[4], ymin, ymax;
   DBL c[3], r[2];
+  DBL *tmp_r1;
+  DBL *tmp_r2;
+  DBL *tmp_h1;
+  DBL *tmp_h2;
   UV_VECT A, B, C, D;
 
   /* Allocate Lathe->Number segments. */
@@ -1222,6 +1161,13 @@ UV_VECT *P;
     Error("Lathe segments are already defined.\n");
   }
 
+  /* Allocate temporary lists. */
+
+  tmp_r1 = (DBL *)POV_MALLOC(Lathe->Number * sizeof(DBL), "temp lathe data");
+  tmp_r2 = (DBL *)POV_MALLOC(Lathe->Number * sizeof(DBL), "temp lathe data");
+  tmp_h1 = (DBL *)POV_MALLOC(Lathe->Number * sizeof(DBL), "temp lathe data");
+  tmp_h2 = (DBL *)POV_MALLOC(Lathe->Number * sizeof(DBL), "temp lathe data");
+
   /***************************************************************************
   * Calculate segments.
   ****************************************************************************/
@@ -1364,19 +1310,24 @@ UV_VECT *P;
 
     /* Set current segment's bounding cylinder. */
 
-    Lathe->Spline->Entry[i].r1 = min(min(x[0], x[1]), min(x[2], x[3]));
-    Lathe->Spline->Entry[i].r2 = max(max(x[0], x[1]), max(x[2], x[3]));
+    tmp_r1[i] = min(min(x[0], x[1]), min(x[2], x[3]));
+    tmp_r2[i] = max(max(x[0], x[1]), max(x[2], x[3]));
 
-    Lathe->Spline->Entry[i].h1 = min(min(y[0], y[1]), min(y[2], y[3]));
-    Lathe->Spline->Entry[i].h2 = max(max(y[0], y[1]), max(y[2], y[3]));
+    tmp_h1[i] = min(min(y[0], y[1]), min(y[2], y[3]));
+    tmp_h2[i] = max(max(y[0], y[1]), max(y[2], y[3]));
 
     /* Keep track of overall bounding cylinder. */
 
-    xmin = min(xmin, Lathe->Spline->Entry[i].r1);
-    xmax = max(xmax, Lathe->Spline->Entry[i].r2);
+    xmin = min(xmin, tmp_r1[i]);
+    xmax = max(xmax, tmp_r2[i]);
 
-    ymin = min(ymin, Lathe->Spline->Entry[i].h1);
-    ymax = max(ymax, Lathe->Spline->Entry[i].h2);
+    ymin = min(ymin, tmp_h1[i]);
+    ymax = max(ymax, tmp_h2[i]);
+
+/*
+    fprintf(stderr, "bound spline segment %d: ", i);
+    fprintf(stderr, "r = %f - %f, h = %f - %f\n", tmp_r1[i], tmp_r2[i], tmp_h1[i], tmp_h2[i]);
+*/
   }
 
   /* Set overall bounding cylinder. */
@@ -1386,6 +1337,75 @@ UV_VECT *P;
 
   Lathe->Height1 = ymin;
   Lathe->Height2 = ymax;
+
+  /* Get bounding cylinder. */
+
+  Lathe->Spline->BCyl = Create_BCyl(Lathe->Number, tmp_r1, tmp_r2, tmp_h1, tmp_h2);
+
+  /* Get rid of temp. memory. */
+
+  POV_FREE(tmp_h2);
+  POV_FREE(tmp_h1);
+  POV_FREE(tmp_r2);
+  POV_FREE(tmp_r1);
+}
+
+
+/*****************************************************************************
+*
+* FUNCTION
+*
+*   test_hit
+*
+* INPUT
+*
+*   Lathe       - Pointer to lathe structure
+*   Ray         - Current ray
+*   Depth_Stack - Current depth stack
+*   d, w, n     - Intersection depth, parameter and segment number
+*
+* OUTPUT
+*
+*   Depth_Stack
+*
+* RETURNS
+*
+* AUTHOR
+*
+*   Dieter Bayer
+*
+* DESCRIPTION
+*
+*   Test if a hit is valid and push if on the intersection depth.
+*
+* CHANGES
+*
+*   Oct 1996 : Creation.
+*
+******************************************************************************/
+
+static int test_hit(Lathe, Ray, Depth_Stack, d, w, n)
+LATHE *Lathe;
+RAY *Ray;
+ISTACK *Depth_Stack;
+DBL d, w;
+int n;
+{
+  VECTOR IPoint;
+
+  if ((d > DEPTH_TOLERANCE) && (d < Max_Distance))
+  {
+    VEvaluateRay(IPoint, Ray->Initial, d, Ray->Direction);
+
+    if (Point_In_Clip(IPoint, ((OBJECT *)Lathe)->Clip))
+    {
+      push_entry_i1_d1(d, IPoint, (OBJECT *)Lathe, n, w, Depth_Stack);
+
+      return(TRUE);
+    }
+  }
+
+  return(FALSE);
 }
 
 
