@@ -689,7 +689,7 @@ char *Option_String;
 
           Option_Number = ANTIALIAS;
 
-          if (sscanf (&Option_String[1], DBL_FORMAT_STRING, &floatval) != EOF)
+          if (sscanf (&Option_String[1], DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
           {
             opts.Antialias_Threshold = floatval;
           }
@@ -922,7 +922,7 @@ char *Option_String;
 
       Option_Number = JITTER;
  
-      if (sscanf (&Option_String[1], DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (&Option_String[1], DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
       {
         opts.JitterScale = floatval;
       }
@@ -1007,7 +1007,7 @@ char *Option_String;
         case 'b': /* Min Bounded */
         case 'B':
 
-          if (sscanf (&Option_String[2], "%ld", &longval) != EOF)
+          if (sscanf (&Option_String[2], "%ld", &longval) != SCANF_EOF)
           {
             opts.BBox_Threshold=longval;
           }
@@ -1278,7 +1278,7 @@ char *value;
   switch (variable)
   {
     case BUF_SIZE_OP:
-      if (sscanf (value, "%d", &opts.File_Buffer_Size) != EOF)
+      if (sscanf (value, "%d", &opts.File_Buffer_Size) != SCANF_EOF)
       {
         opts.File_Buffer_Size *= 1024;
 
@@ -1370,12 +1370,12 @@ char *value;
       break;
 
     case ANTIALIAS_THRESH_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         opts.Antialias_Threshold = floatval;
       return;
 
     case ANTIALIAS_DEPTH_OP:
-      if (sscanf (value, "%ld", &longval) != EOF)
+      if (sscanf (value, "%ld", &longval) != SCANF_EOF)
         opts.AntialiasDepth = longval;
       if (opts.AntialiasDepth < 1)
         opts.AntialiasDepth = 1;
@@ -1388,7 +1388,7 @@ char *value;
       break;
 
     case JITTER_AMOUNT_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         opts.JitterScale = floatval;
       if (opts.JitterScale<=0.0)
         opts.Options &= ~JITTER;
@@ -1415,7 +1415,7 @@ char *value;
       return;
 
     case START_COLUMN_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         if(floatval > 0.0 && floatval < 1.0)
         {
           opts.First_Column = -1;
@@ -1429,7 +1429,7 @@ char *value;
       return;
 
     case START_ROW_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         if(floatval > 0.0 && floatval < 1.0)
         {
           opts.First_Line = -1;
@@ -1443,7 +1443,7 @@ char *value;
       return;
 
     case END_COLUMN_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         if(floatval > 0.0 && floatval <= 1.0)
         {
           opts.Last_Column = -1;
@@ -1454,7 +1454,7 @@ char *value;
       return;
 
     case END_ROW_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         if(floatval > 0.0 && floatval <= 1.0)
         {
           opts.Last_Line = -1;
@@ -1465,7 +1465,7 @@ char *value;
       return;
 
     case VERSION_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
         opts.Language_Version = floatval;
       return;
 
@@ -1496,42 +1496,42 @@ char *value;
       return;
 
     case CLOCK_OP:
-      if(sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if(sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
       {
         opts.FrameSeq.Clock_Value = floatval;
       }
       return;
 
     case INITIAL_FRAME_OP:
-      if(sscanf(value, "%ld", &longval)!=EOF)
+      if(sscanf(value, "%ld", &longval)!=SCANF_EOF)
       {
         opts.FrameSeq.InitialFrame=longval;
       }
       return;
 
     case INITIAL_CLOCK_OP:
-      if(sscanf(value, DBL_FORMAT_STRING, &floatval)!=EOF)
+      if(sscanf(value, DBL_FORMAT_STRING, &floatval)!=SCANF_EOF)
       {
         opts.FrameSeq.InitialClock=floatval;
       }
       return;
 
     case FINAL_FRAME_OP:
-      if(sscanf(value, "%ld", &longval)!=EOF)
+      if(sscanf(value, "%ld", &longval)!=SCANF_EOF)
       {
         opts.FrameSeq.FinalFrame=longval;
       }
       return;
 
     case FINAL_CLOCK_OP:
-      if(sscanf(value, DBL_FORMAT_STRING, &floatval)!=EOF)
+      if(sscanf(value, DBL_FORMAT_STRING, &floatval)!=SCANF_EOF)
       {
         opts.FrameSeq.FinalClock=floatval;
       }
       return;
 
     case SUBSET_START_FRAME_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
       {
         if(floatval > 0.0 && floatval < 1.0)
           opts.FrameSeq.SubsetStartPercent=floatval;
@@ -1541,7 +1541,7 @@ char *value;
       return;
 
     case SUBSET_END_FRAME_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
       {
         if(floatval > 0.0 && floatval < 1.0)
           opts.FrameSeq.SubsetEndPercent=floatval;
@@ -1620,7 +1620,7 @@ char *value;
       break;
 
     case HIST_SIZE_OP:
-      if (sscanf (value, "%d.%d", &opts.histogram_x, &opts.histogram_y) == EOF)
+      if (sscanf (value, "%d.%d", &opts.histogram_x, &opts.histogram_y) == SCANF_EOF)
       {
         Warning (0.0, "Error occurred scanning histogram grid size '%s'.\n", value) ;
         opts.histogram_on = FALSE ;
@@ -1812,7 +1812,7 @@ char *value;
       return;
 
     case DISPLAY_GAMMA_OP:
-      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != EOF)
+      if (sscanf (value, DBL_FORMAT_STRING, &floatval) != SCANF_EOF)
       {
         if (floatval > 0.0)
           opts.DisplayGamma = floatval;
