@@ -112,11 +112,11 @@ static void Close_Png_File PARAMS((FILE_HANDLE *handle));
 
 
 /* These are replacement error and warning functions for the libpng code */
-static void png_pov_err PARAMS((png_struct *png_ptr, char *msg));
-static void png_pov_warn PARAMS((png_struct *png_ptr, char *msg));
+static void png_pov_err PARAMS((png_structp, png_const_charp));
+static void png_pov_warn PARAMS((png_structp, png_const_charp));
 
 /* This is an internal function for libpng */
-extern void png_write_finish_row PARAMS((png_struct *png_ptr));
+extern void png_write_finish_row PARAMS((png_structp));
 
 /*****************************************************************************
 *
@@ -183,8 +183,8 @@ FILE_HANDLE *Get_Png_File_Handle()
 *
 ******************************************************************************/
 static void png_pov_warn(png_ptr, msg)
-png_struct *png_ptr;
-char *msg;
+png_structp png_ptr;
+png_const_charp msg;
 {
   if (png_get_error_ptr(png_ptr))
     Warning(0.0,"libpng: %s\n",msg);
@@ -213,8 +213,8 @@ char *msg;
 *
 ******************************************************************************/
 static void png_pov_err(png_ptr, msg)
-png_struct *png_ptr;
-char *msg;
+png_structp png_ptr;
+png_const_charp msg;
 {
   if (png_get_error_ptr(png_ptr))
     Error_Line("libpng: %s\n",msg);
