@@ -3809,7 +3809,11 @@ int Compute;
       {
         if (Inside(Ray->Initial, Object))
         {
-          Containing_Objects[++Containing_Index] = Object;
+          if ((++Containing_Index)>=MAX_CONTAINING_OBJECTS)
+          {
+             Error("Too many nested objects\n");
+          }
+          Containing_Objects[Containing_Index] = Object;
           Containing_Textures[Containing_Index]  = Object->Texture;
 
           if ((Object->Texture != NULL) &&
@@ -3934,7 +3938,11 @@ BBOX_TREE *Node;
     {
       if (Object->Texture != NULL)
       {
-        Containing_Objects[++Containing_Index] = Object;
+        if ((++Containing_Index)>=MAX_CONTAINING_OBJECTS)
+        {
+           Error("Too many nested objects\n");
+        }
+        Containing_Objects[Containing_Index] = Object;
         Containing_Textures[Containing_Index]  = Object->Texture;
 
         if ((Object->Texture != NULL) &&
