@@ -113,27 +113,23 @@
 #define CAP_PLANE  2
 #define CURVE      3
 
+/* Max. number of intersecions per spline segment. */
+
+#define MAX_INTERSECTIONS_PER_SEGMENT 4
+
 
 
 /*****************************************************************************
 * Local typedefs
 ******************************************************************************/
 
-typedef struct Sor_Intersection_Structure SOR_INT;
-
-struct Sor_Intersection_Structure
-{
-  DBL d;
-  int t, n;
-};
-
 
 
 /*****************************************************************************
 * Static functions
 ******************************************************************************/
 
-static int intersect_sor PARAMS((RAY *Ray, SOR *Sor, SOR_INT *Int));
+static int  intersect_sor PARAMS((RAY *Ray, SOR *Sor, ISTACK *Depth_Stack));
 static int  All_Sor_Intersections PARAMS((OBJECT *Object, RAY *Ray, ISTACK *Depth_Stack));
 static int  Inside_Sor PARAMS((VECTOR point, OBJECT *Object));
 static void Sor_Normal PARAMS((VECTOR Result, OBJECT *Object, INTERSECTION *Inter));
@@ -145,6 +141,7 @@ static void Transform_Sor PARAMS((OBJECT *Object, TRANSFORM *Trans));
 static void Invert_Sor PARAMS((OBJECT *Object));
 static void Destroy_Sor PARAMS((OBJECT *Object));
 
+static int test_hit PARAMS((SOR *, RAY *, ISTACK *, DBL, int, int));
 
 /*****************************************************************************
 * Local variables
@@ -172,7 +169,7 @@ static METHODS Sor_Methods =
 *   Object      - Object
 *   Ray         - Ray
 *   Depth_Stack - Intersection stack
-*   
+*
 * OUTPUT
 *
 *   Depth_Stack
@@ -193,6 +190,7 @@ static METHODS Sor_Methods =
 * CHANGES
 *
 *   May 1994 : Creation.
+*   Oct 1996 : Changed code to include faster version. [DB]
 *
 ******************************************************************************/
 
@@ -201,36 +199,16 @@ OBJECT *Object;
 RAY *Ray;
 ISTACK *Depth_Stack;
 {
-  int i, max_i, Found;
-  SOR_INT *I;
-  VECTOR IPoint;
+  Increase_Counter(stats[Ray_Sor_Tests]);
 
-  /* Allocate memory for intersections. */
-
-  I = (SOR_INT *)POV_MALLOC(5*((SOR *)Object)->Number*sizeof(SOR_INT), "surface of revolution intersection array");
-
-  Found = FALSE;
-
-  max_i = intersect_sor(Ray, (SOR *)Object, I);
-
-  if (max_i)
+  if (intersect_sor(Ray, (SOR *)Object, Depth_Stack))
   {
-    for (i = 0; i < max_i; i++)
-    {
-      VEvaluateRay(IPoint, Ray->Initial, I[i].d, Ray->Direction);
+    Increase_Counter(stats[Ray_Sor_Tests_Succeeded]);
 
-      if (Point_In_Clip(IPoint, Object->Clip))
-      {
-        push_entry_i1_i2(I[i].d,IPoint,Object,I[i].t,I[i].n,Depth_Stack);
-
-        Found = TRUE;
-      }
-    }
+    return(TRUE);
   }
 
-  POV_FREE (I);
-
-  return(Found);
+  return(FALSE);
 }
 
 
@@ -254,11 +232,11 @@ ISTACK *Depth_Stack;
 * RETURNS
 *
 *   int - Number of intersections found
-*   
+*
 * AUTHOR
 *
 *   Dieter Bayer
-*   
+*
 * DESCRIPTION
 *
 *   Determine ray/surface of revolution intersection.
@@ -269,26 +247,23 @@ ISTACK *Depth_Stack;
 * CHANGES
 *
 *   May 1994 : Creation.
+*   Oct 1996 : Changed code to include faster version. [DB]
 *
 ******************************************************************************/
 
-static int intersect_sor(Ray, Sor, Intersection)
+static int intersect_sor(Ray, Sor, Depth_Stack)
 RAY *Ray;
 SOR *Sor;
-SOR_INT *Intersection;
+ISTACK *Depth_Stack;
 {
-  int i = 0, j, n;
-  DBL a, b, k, l, len, u, v, x[4], y[3], r0;
+  int cnt;
+  int found, j, n;
+  DBL a, b, k, h, len, u, v, r0;
+  DBL x[4], y[3];
+  DBL best;
   VECTOR P, D;
-  SOR_SPLINE_ENTRY Entry;
-
-  Increase_Counter(stats[Ray_Sor_Tests]);
-
-  /* Init intersection structure. */
-
-  Intersection->d = 0.0;
-  Intersection->n =
-  Intersection->t = 0;
+  BCYL_INT *intervals;
+  SOR_SPLINE_ENTRY *Entry;
 
   /* Transform the ray into the surface of revolution space. */
 
@@ -330,98 +305,105 @@ SOR_INT *Intersection;
     return(FALSE);
   }
 
+  /* Intersect all cylindrical bounds. */
+
+  if ((cnt = Intersect_BCyl(Sor->Spline->BCyl, P, D)) == 0)
+  {
+    return(FALSE);
+  }
+
 #ifdef SOR_EXTRA_STATS
   Increase_Counter(stats[Sor_Bound_Tests_Succeeded]);
 #endif
 
-  if (fabs(D[Y]) > DEPTH_TOLERANCE)
+  /* Test base/cap plane. */
+
+  found = FALSE;
+
+  best = BOUND_HUGE;
+
+  if (Test_Flag(Sor, CLOSED_FLAG) && (fabs(D[Y]) > EPSILON))
   {
-    /* Test base/cap plane. */
+    /* Test base plane. */
 
-    if (Test_Flag(Sor, CLOSED_FLAG))
+    if (Sor->Base_Radius_Squared > DEPTH_TOLERANCE)
     {
-      /* Test base plane. */
+      k = (Sor->Height1 - P[Y]) / D[Y];
 
-      if (Sor->Base_Radius_Squared > DEPTH_TOLERANCE)
+      u = P[X] + k * D[X];
+      v = P[Z] + k * D[Z];
+
+      b = u * u + v * v;
+
+      if (b <= Sor->Base_Radius_Squared)
       {
-        a = (Sor->Height1 - P[Y]) / D[Y];
-
-        if ((a > DEPTH_TOLERANCE) && (a < Max_Distance))
+        if (test_hit(Sor, Ray, Depth_Stack, k / len, BASE_PLANE, 0))
         {
-          u = P[X] + a * D[X];
-          v = P[Z] + a * D[Z];
+          found = TRUE;
 
-          b = u * u + v * v;
-
-          if (b <= Sor->Base_Radius_Squared)
+          if (k < best)
           {
-            Intersection[i].t   = BASE_PLANE;
-            Intersection[i++].d = a / len;
+            best = k;
           }
         }
       }
+    }
 
-      /* Test cap plane. */
+    /* Test cap plane. */
 
-      if (Sor->Cap_Radius_Squared > DEPTH_TOLERANCE)
+    if (Sor->Cap_Radius_Squared > DEPTH_TOLERANCE)
+    {
+      k = (Sor->Height1 - P[Y]) / D[Y];
+
+      u = P[X] + k * D[X];
+      v = P[Z] + k * D[Z];
+
+      b = u * u + v * v;
+
+      if (b <= Sor->Cap_Radius_Squared)
       {
-        a = (Sor->Height1 - P[Y]) / D[Y];
-
-        if ((a > DEPTH_TOLERANCE) && (a < Max_Distance))
+        if (test_hit(Sor, Ray, Depth_Stack, k / len, CAP_PLANE, 0))
         {
-          u = P[X] + a * D[X];
-          v = P[Z] + a * D[Z];
+          found = TRUE;
 
-          b = u * u + v * v;
-
-          if (b <= Sor->Cap_Radius_Squared)
+          if (k < best)
           {
-            Intersection[i].t   = BASE_PLANE;
-            Intersection[i++].d = a / len;
+            best = k;
           }
         }
       }
     }
   }
 
-  /* Test all segments. */
+  /* Step through the list of intersections. */
 
-  for (j = 0; j < Sor->Number; j++)
+  intervals = Sor->Spline->BCyl->intervals;
+
+  for (j = 0; j < cnt; j++)
   {
-    Entry = Sor->Spline->Entry[j];
+    /* Get current segment. */
 
-    /* Test if ray misses segment's bounds. */
+    Entry = &Sor->Spline->Entry[intervals[j].n];
 
-#ifdef SOR_EXTRA_STATS
-    Increase_Counter(stats[Sor_Bound_Tests]);
-#endif
+    /* If we already have the best intersection we may exit. */
 
-    if (((D[Y] >= 0.0) && (P[Y] >  Entry.h2)) ||
-        ((D[Y] <= 0.0) && (P[Y] <  Entry.h1)) ||
-        ((D[X] >= 0.0) && (P[X] >  Entry.r2)) ||
-        ((D[X] <= 0.0) && (P[X] < -Entry.r2)))
+    if (!(Sor->Type & IS_CHILD_OBJECT) && (intervals[j].d[0] > best))
     {
-      continue;
+      break;
     }
 
-    if (r0 > Entry.r2)
-    {
-      continue;
-    }
-
-#ifdef SOR_EXTRA_STATS
-    Increase_Counter(stats[Sor_Bound_Tests_Succeeded]);
-#endif
-
     /* Cubic curve. */
 
-    x[0] = Entry.A * D[Y] * D[Y] * D[Y];
+    x[0] = Entry->A * D[Y] * D[Y] * D[Y];
 
-    x[1] = D[Y] * D[Y] * (3.0 * Entry.A * P[Y] + Entry.B) - D[X] * D[X] - D[Z] * D[Z];
+/*
+    x[1] = D[Y] * D[Y] * (3.0 * Entry->A * P[Y] + Entry->B) - D[X] * D[X] - D[Z] * D[Z];
+*/
+    x[1] = D[Y] * D[Y] * (3.0 * Entry->A * P[Y] + Entry->B) - a;
 
-    x[2] = D[Y] * (P[Y] * (3.0 * Entry.A * P[Y] + 2.0 * Entry.B) + Entry.C) - 2.0 * (P[X] * D[X] + P[Z] * D[Z]);
+    x[2] = D[Y] * (P[Y] * (3.0 * Entry->A * P[Y] + 2.0 * Entry->B) + Entry->C) - 2.0 * (P[X] * D[X] + P[Z] * D[Z]);
 
-    x[3] = P[Y] * (P[Y] * (Entry.A * P[Y] + Entry.B) + Entry.C) + Entry.D - P[X] * P[X] - P[Z] * P[Z];
+    x[3] = P[Y] * (P[Y] * (Entry->A * P[Y] + Entry->B) + Entry->C) + Entry->D - P[X] * P[X] - P[Z] * P[Z];
 
     n = Solve_Polynomial(3, x, y, Test_Flag(Sor, STURM_FLAG), 0.0);
 
@@ -429,26 +411,25 @@ SOR_INT *Intersection;
     {
       k = y[n];
 
-      if ((k > DEPTH_TOLERANCE) && (k < Max_Distance))
-      {
-        l = P[Y] + k * D[Y];
+      h = P[Y] + k * D[Y];
 
-        if ((l >= Entry.h1) && (l <= Entry.h2))
+      if ((h >= Sor->Spline->BCyl->height[Sor->Spline->BCyl->entry[intervals[j].n].h1]) &&
+          (h <= Sor->Spline->BCyl->height[Sor->Spline->BCyl->entry[intervals[j].n].h2]))
+      {
+        if (test_hit(Sor, Ray, Depth_Stack, k / len, CURVE, intervals[j].n))
         {
-          Intersection[i].t   = CURVE;
-          Intersection[i].n   = j;
-          Intersection[i++].d = k / len;
+          found = TRUE;
+
+          if (y[n] < best)
+          {
+            best = k;
+          }
         }
       }
     }
   }
 
-  if (i)
-  {
-    Increase_Counter(stats[Ray_Sor_Tests_Succeeded]);
-  }
-
-  return(i);
+  return(found);
 }
 
 
@@ -492,7 +473,7 @@ OBJECT *Object;
   DBL r0, r;
   VECTOR P;
   SOR *Sor = (SOR *)Object;
-  SOR_SPLINE_ENTRY Entry;
+  SOR_SPLINE_ENTRY *Entry;
 
   /* Transform the point into the surface of revolution space. */
 
@@ -512,11 +493,10 @@ OBJECT *Object;
 
       for (i = 0; i < Sor->Number; i++)
       {
-        Entry = Sor->Spline->Entry[i];
+        Entry = &Sor->Spline->Entry[i];
 
-        /* P[Y] >= R->Entry[i].h1 needn't be tested. */
-
-        if ((P[Y] >= Entry.h1) && (P[Y] <= Entry.h2))
+        if ((P[Y] >= Sor->Spline->BCyl->height[Sor->Spline->BCyl->entry[i].h1]) &&
+            (P[Y] <= Sor->Spline->BCyl->height[Sor->Spline->BCyl->entry[i].h2]))
         {
           break;
         }
@@ -526,7 +506,7 @@ OBJECT *Object;
 
       if (i < Sor->Number)
       {
-        r = P[Y] * (P[Y] * (P[Y] * Entry.A + Entry.B) + Entry.C) + Entry.D;
+        r = P[Y] * (P[Y] * (P[Y] * Entry->A + Entry->B) + Entry->C) + Entry->D;
 
         if (r0 <= r)
         {
@@ -585,30 +565,45 @@ INTERSECTION *Inter;
   DBL k;
   VECTOR P;
   SOR *Sor = (SOR *)Object;
-  SOR_SPLINE_ENTRY Entry;
+  SOR_SPLINE_ENTRY *Entry;
   VECTOR N;
 
-  Make_Vector(N, 0.0, 1.0, 0.0);
-
-  if (Inter->i1 == CURVE)
+  switch (Inter->i1)
   {
-    /* Transform the intersection point into the surface of revolution space. */
+    case CURVE:
 
-    MInvTransPoint(P, Inter->IPoint, Sor->Trans);
+      /* Transform the intersection point into the surface of revolution space. */
 
-    if (P[X] * P[X] + P[Z] * P[Z] > DEPTH_TOLERANCE)
-    {
-      Entry = Sor->Spline->Entry[Inter->i2];
+      MInvTransPoint(P, Inter->IPoint, Sor->Trans);
 
-      k = 0.5 * (P[Y] * (3.0 * Entry.A * P[Y] + 2.0 * Entry.B) + Entry.C);
+      if (P[X] * P[X] + P[Z] * P[Z] > DEPTH_TOLERANCE)
+      {
+        Entry = &Sor->Spline->Entry[Inter->i2];
 
-      N[X] = P[X];
-      N[Y] = -k;
-      N[Z] = P[Z];
-    }
+        k = 0.5 * (P[Y] * (3.0 * Entry->A * P[Y] + 2.0 * Entry->B) + Entry->C);
+
+        N[X] = P[X];
+        N[Y] = -k;
+        N[Z] = P[Z];
+      }
+
+      break;
+
+    case BASE_PLANE:
+
+      Make_Vector(N, 0.0, -1.0, 0.0);
+
+      break;
+
+
+    case CAP_PLANE:
+
+      Make_Vector(N, 0.0, 1.0, 0.0);
+
+      break;
   }
 
-  /* Transform the normalt out of the surface of revolution space. */
+  /* Transform the normal out of the surface of revolution space. */
 
   MTransNormal(Result, N, Sor->Trans);
 
@@ -969,12 +964,14 @@ OBJECT *Object;
 
   if (--(Sor->Spline->References) == 0)
   {
-    POV_FREE (Sor->Spline->Entry);
+    Destroy_BCyl(Sor->Spline->BCyl);
 
-    POV_FREE (Sor->Spline);
+    POV_FREE(Sor->Spline->Entry);
+
+    POV_FREE(Sor->Spline);
   }
 
-  POV_FREE (Object);
+  POV_FREE(Object);
 }
 
 
@@ -1059,8 +1056,12 @@ SOR *Sor;
 UV_VECT *P;
 {
   int i, n;
+  DBL *tmp_r1;
+  DBL *tmp_r2;
+  DBL *tmp_h1;
+  DBL *tmp_h2;
   DBL A, B, C, D, w, k[4];
-  DBL x[4], xmax;
+  DBL x[4], xmax, xmin;
   DBL y[2], ymax, ymin;
   DBL c[3], r[2];
   MATRIX Mat;
@@ -1071,7 +1072,7 @@ UV_VECT *P;
   {
     Sor->Spline = (SOR_SPLINE *)POV_MALLOC(sizeof(SOR_SPLINE), "spline segments of surface of revoluion");
 
-      Sor->Spline->References = 1;
+    Sor->Spline->References = 1;
 
     Sor->Spline->Entry = (SOR_SPLINE_ENTRY *)POV_MALLOC(Sor->Number*sizeof(SOR_SPLINE_ENTRY), "spline segments of surface of revoluion");
   }
@@ -1080,10 +1081,17 @@ UV_VECT *P;
     Error("Surface of revolution segments are already defined.\n");
   }
 
+  /* Allocate temporary lists. */
+
+  tmp_r1 = (DBL *)POV_MALLOC(Sor->Number * sizeof(DBL), "temp lathe data");
+  tmp_r2 = (DBL *)POV_MALLOC(Sor->Number * sizeof(DBL), "temp lathe data");
+  tmp_h1 = (DBL *)POV_MALLOC(Sor->Number * sizeof(DBL), "temp lathe data");
+  tmp_h2 = (DBL *)POV_MALLOC(Sor->Number * sizeof(DBL), "temp lathe data");
+
   /* We want to know the size of the overall bounding cylinder. */
 
   xmax = ymax = -BOUND_HUGE;
-         ymin =  BOUND_HUGE;
+  xmin = ymin =  BOUND_HUGE;
 
   /* Calculate segments, i.e. cubic patches. */
 
@@ -1148,7 +1156,7 @@ UV_VECT *P;
     Sor->Spline->Entry[i].C = C;
     Sor->Spline->Entry[i].D = D;
 
-    /* Get maximum radius**2 in current segment. */
+    /* Get minimum and maximum radius**2 in current segment. */
 
     y[0] = P[i+1][Y];
     y[1] = P[i+2][Y];
@@ -1172,21 +1180,29 @@ UV_VECT *P;
 
     /* Set current segment's bounding cylinder. */
 
-    Sor->Spline->Entry[i].h1 = y[0];
-    Sor->Spline->Entry[i].h2 = y[1];
+    tmp_r1[i] = min(min(x[0], x[1]), min(x[2], x[3]));
+    tmp_r2[i] = max(max(x[0], x[1]), max(x[2], x[3]));
 
-    Sor->Spline->Entry[i].r2 = max(max(x[0], x[1]), max(x[2], x[3]));
+    tmp_h1[i] = y[0];
+    tmp_h2[i] = y[1];
 
     /* Keep track of overall bounding cylinder. */
 
-    xmax = max(xmax, Sor->Spline->Entry[i].r2);
+    xmin = min(xmin, tmp_r1[i]);
+    xmax = max(xmax, tmp_r2[i]);
 
-    ymin = min(ymin, Sor->Spline->Entry[i].h1);
-    ymax = max(ymax, Sor->Spline->Entry[i].h2);
+    ymin = min(ymin, tmp_h1[i]);
+    ymax = max(ymax, tmp_h2[i]);
+
+/*
+    fprintf(stderr, "bound spline segment %d: ", i);
+    fprintf(stderr, "r = %f - %f, h = %f - %f\n", tmp_r1[i], tmp_r2[i], tmp_h1[i], tmp_h2[i]);
+*/
   }
 
   /* Set overall bounding cylinder. */
 
+  Sor->Radius1 = ymax;
   Sor->Radius2 = xmax;
 
   Sor->Height1 = ymin;
@@ -1194,7 +1210,7 @@ UV_VECT *P;
 
   /* Get cap radius. */
 
-  w = Sor->Spline->Entry[Sor->Number-1].h1;
+  w = tmp_h1[Sor->Number-1];
 
   A = Sor->Spline->Entry[Sor->Number-1].A;
   B = Sor->Spline->Entry[Sor->Number-1].B;
@@ -1208,7 +1224,7 @@ UV_VECT *P;
 
   /* Get base radius. */
 
-  w = Sor->Spline->Entry[0].h1;
+  w = tmp_h1[0];
 
   A = Sor->Spline->Entry[0].A;
   B = Sor->Spline->Entry[0].B;
@@ -1219,8 +1235,77 @@ UV_VECT *P;
   {
     Sor->Base_Radius_Squared = 0.0;
   }
+
+  /* Get bounding cylinder. */
+
+  Sor->Spline->BCyl = Create_BCyl(Sor->Number, tmp_r1, tmp_r2, tmp_h1, tmp_h2);
+
+  /* Get rid of temp. memory. */
+
+  POV_FREE(tmp_h2);
+  POV_FREE(tmp_h1);
+  POV_FREE(tmp_r2);
+  POV_FREE(tmp_r1);
 }
 
 
 
+/*****************************************************************************
+*
+* FUNCTION
+*
+*   test_hit
+*
+* INPUT
+*
+*   Sor         - Pointer to lathe structure
+*   Ray         - Current ray
+*   Depth_Stack - Current depth stack
+*   d, t, n     - Intersection depth, type and number
+*
+* OUTPUT
+*
+*   Depth_Stack
+*
+* RETURNS
+*
+* AUTHOR
+*
+*   Dieter Bayer
+*
+* DESCRIPTION
+*
+*   Test if a hit is valid and push if on the intersection depth.
+*
+* CHANGES
+*
+*   Oct 1996 : Creation.
+*
+******************************************************************************/
+
+static int test_hit(Sor, Ray, Depth_Stack, d, t, n)
+SOR *Sor;
+RAY *Ray;
+ISTACK *Depth_Stack;
+DBL d;
+int t, n;
+{
+  VECTOR IPoint;
+
+  if ((d > DEPTH_TOLERANCE) && (d < Max_Distance))
+  {
+    VEvaluateRay(IPoint, Ray->Initial, d, Ray->Direction);
+
+    if (Point_In_Clip(IPoint, ((OBJECT *)Sor)->Clip))
+    {
+      push_entry_i1_i2(d, IPoint, (OBJECT *)Sor, t, n, Depth_Stack);
+
+      return(TRUE);
+    }
+  }
+
+  return(FALSE);
+}
+
+
 
