@@ -261,7 +261,9 @@ int mode;
       switch(opts.OutputFormat)  /* Byte 2 - TGA file type */
       {
         case 't': /* Uncompressed True-Color Image */
-        case 'T': putc(2, handle->file); break;
+        case 'T': 
+        case 's': 
+        case 'S': putc(2, handle->file); break;
 
         case 'c': /* Run-length compressed True-Color Image */
         case 'C': putc(10, handle->file); break;
@@ -454,6 +456,8 @@ int line_number;
   {
     case 't':
     case 'T':
+    case 's':
+    case 'S':
 
       for (x = 0; x < handle->width; x++)
       {
@@ -626,6 +630,8 @@ int *line_number;
   {
     case 't':
     case 'T':
+    case 's':
+    case 'S':
 
       for (x = 0; x < handle->width; x++)
       {
