@@ -35,7 +35,8 @@
 * Added support for multiple glyph characters, Feb 1996 [AED]
 * Changed functions to only use promoted types (no USHORTs), Mar 1996 [AED]
 * Cache Format 4 glyph table index to speed up parsing, Mar 1996 [AED]
-*
+* Fixed TTF rendering bug that leaves horizontal streaks, Nov. 1996  Kochin Chang (KochinC@aol.com)
+* Allowed Macintosh-charmap TTF files to be processed, Nov. 1996 Bob Spence/Eduard Schwan
 *****************************************************************************/
 
 #include "frame.h"
@@ -1175,7 +1176,8 @@ long kern_table_offset;
   /* Read in the number of kerning tables */
 
   kern_table->nTables = READUSHORT(ffile->fp);
-
+  kern_table->tables = NULL;      /*<==[esp] added (in case nTables is zero)*/
+  
 #ifdef TTF_DEBUG
   Debug_Info("\nKerning table:\n", kern_table_offset);
   Debug_Info("Offset: %ld\n", kern_table_offset);
@@ -1211,6 +1213,9 @@ long kern_table_offset;
                          " Override" : "" ));
 #endif
 
+    kern_table->tables[i].kern_pairs = NULL;   /*<==[esp] added*/
+    kern_table->tables[i].nPairs = 0;          /*<==[esp] added*/
+
     if ((kern_table->tables[i].coverage >> 8) == 0)
     {
       /* Can only handle format 0 kerning subtables */
@@ -1446,7 +1451,7 @@ unsigned int *glyph_index;
 *
 * CHANGES
 *
-*   -
+*   961120  esp  Added check to allow Macintosh encodings to pass
 *
 ******************************************************************************/
 static USHORT ProcessCharMap(ffile, search_char)
@@ -1489,8 +1494,9 @@ unsigned int search_char;
      * Check if this is the encoding table we want to use.  We can't
      * be very picky, since we don't know what the user really wants.
      * It would be nice to know what we are really looking for.
+	 * Allow Type 1 platform as well (Macintosh) [esp]
      */
-    if (ffile->platformID != cmapEntry.platformID)
+    if ((ffile->platformID != cmapEntry.platformID) && (1 != cmapEntry.platformID))
     {
       continue;
     }
@@ -2494,11 +2500,40 @@ GlyphPtr glyph;
           }
         }
 
+        /* only test crossing when y is in the range */
+        /* this should also help saving some computations */
+        if (((y0 < y) && (y1 < y) && (y2 < y)) ||
+            ((y0 > y) && (y1 > y) && (y2 > y)))
+          goto end_curve_test;
+
         yt[0] = y0 - 2.0 * y1 + y2;
         yt[1] = 2.0 * (y1 - y0);
         yt[2] = y0 - y;
         
-        k = solve_quad(yt, roots, EPSILON, 1.0);
+        k = solve_quad(yt, roots, 0.0, 1.0);
+
+        for (ri = 0; ri < k;) {
+          if (roots[ri] <= EPSILON) {
+            /* if y actually is not in range, discard the root */
+            if (((y <= y0) && (y < y1)) || ((y >= y0) && (y > y1))) {
+              k--;
+              if (k > ri)
+                roots[ri] = roots[ri+1];
+              continue;
+            }
+          }
+         else if (roots[ri] >= (1.0 - EPSILON)) {
+            /* if y actually is not in range, discard the root */
+            if (((y < y2) && (y < y1)) || ((y > y2) && (y > y1))) {
+              k--;
+              if (k > ri)
+                roots[ri] = roots[ri+1];
+              continue;
+            }
+          }
+
+          ri++;
+        }
         
         if (k > 0)
         {
@@ -2522,6 +2557,8 @@ GlyphPtr glyph;
               crossings++;
           }
         }
+
+end_curve_test:
         
         x0 = x2;
         
@@ -2723,7 +2760,7 @@ ISTACK *Depth_Stack;
         Point_In_Clip(IPoint, Object->Clip))
     {
       Make_Vector(N, 0.0, 0.0, -1.0);
-      MInvTransNormal(N, N, ttf->Trans);
+      MTransNormal(N, N, ttf->Trans);
       VNormalize(N, N);
       push_normal_entry(Depth, IPoint, N, Object, Depth_Stack);
       Flag = TRUE;
@@ -2740,7 +2777,7 @@ ISTACK *Depth_Stack;
         Point_In_Clip(IPoint, Object->Clip))
     {
       Make_Vector(N, 0.0, 0.0, 1.0);
-      MInvTransNormal(N, N, ttf->Trans);
+      MTransNormal(N, N, ttf->Trans);
       VNormalize(N, N);
       push_normal_entry(Depth, IPoint, N, Object, Depth_Stack);
       Flag = TRUE;
@@ -2828,7 +2865,7 @@ ISTACK *Depth_Stack;
           if (Point_In_Clip(IPoint, Object->Clip))
           {
             Make_Vector(N, d1, -d0, 0.0);
-            MInvTransNormal(N, N, ttf->Trans);
+            MTransNormal(N, N, ttf->Trans);
             VNormalize(N, N);
             push_normal_entry(Depth, IPoint, N, Object, Depth_Stack);
             Flag = TRUE;
@@ -2902,7 +2939,7 @@ ISTACK *Depth_Stack;
             if (Point_In_Clip(IPoint, Object->Clip))
             {
               Make_Vector(N, 2.0 * yt2 * S[l] + yt1, -2.0 * xt2 * S[l] - xt1, 0.0);
-              MInvTransNormal(N, N, ttf->Trans);
+              MTransNormal(N, N, ttf->Trans);
               VNormalize(N, N);
               push_normal_entry(Depth, IPoint, N, Object, Depth_Stack);
               Flag = TRUE;
