@@ -154,7 +154,6 @@ static void POV_Std_Display_Plot_Box PARAMS((int x1,int y1,int x2,int y2,
 void Prune_Vista_Tree(y)
 int y;
 {
-  unsigned size;
   unsigned short i;
   PROJECT_TREE_NODE *Node, *Sib;
 
@@ -165,7 +164,7 @@ int y;
     return;
   }
 
-  size = 0;
+  Node_Queue->QSize = 0;
 
   Increase_Counter(stats[VBuffer_Tests]);
 
@@ -183,12 +182,12 @@ int y;
 
     Root_Vista->is_leaf &= ~PRUNE_TEMPORARY;
 
-    Node_Queue->Queue[size++] = Root_Vista;
+    Node_Queue->Queue[(Node_Queue->QSize)++] = Root_Vista;
   }
 
-  while (size > 0)
+  while (Node_Queue->QSize > 0)
   {
-    Node = Node_Queue->Queue[--size];
+    Node = Node_Queue->Queue[--(Node_Queue->QSize)];
 
     if (Node->is_leaf & TRUE)
     {
@@ -236,10 +235,10 @@ int y;
           /* Add sibling to list */
 
           /* Reallocate queue if it's too small. */
-
+          
           Reinitialize_VLBuffer_Code();
 
-          Node_Queue->Queue[size++] = Sib;
+          Node_Queue->Queue[(Node_Queue->QSize)++] = Sib;
         }
       }
     }
@@ -431,7 +430,7 @@ INTERSECTION *Best_Intersection;
 
   /* Descend tree. */
 
-  size = 0;
+  Node_Queue->QSize = 0;
 
   /* Create the direction vectors for this ray */
 
@@ -447,12 +446,12 @@ INTERSECTION *Best_Intersection;
   {
     Increase_Counter(stats[VBuffer_Tests_Succeeded]);
 
-    Node_Queue->Queue[size++] = Tree;
+    Node_Queue->Queue[(Node_Queue->QSize)++] = Tree;
   }
 
-  while (size > 0)
+  while (Node_Queue->QSize > 0)
   {
-    Tree = Node_Queue->Queue[--size];
+    Tree = Node_Queue->Queue[--(Node_Queue->QSize)];
 
     switch (Tree->is_leaf)
     {
@@ -480,7 +479,7 @@ INTERSECTION *Best_Intersection;
 
               Reinitialize_VLBuffer_Code();
 
-              Node_Queue->Queue[size++] = Node;
+              Node_Queue->Queue[(Node_Queue->QSize)++] = Node;
             }
           }
         }
@@ -854,11 +853,9 @@ int *visible;
 
   for (i = 0; i<8; i++)
   {
-    Assign_BBox_Vect(P[i], BBox->Lower_Left);
-
-    P[i][X] += ((i & 1) ? BBox->Lengths[X] : 0.0);
-    P[i][Y] += ((i & 2) ? BBox->Lengths[Y] : 0.0);
-    P[i][Z] += ((i & 4) ? BBox->Lengths[Z] : 0.0);
+    P[i][X] = ((i & 1) ? BBox->Lengths[X] : 0.0) + BBox->Lower_Left[X];
+    P[i][Y] = ((i & 2) ? BBox->Lengths[Y] : 0.0) + BBox->Lower_Left[Y];
+    P[i][Z] = ((i & 4) ? BBox->Lengths[Z] : 0.0) + BBox->Lower_Left[Z];
 
     transform_point(P[i]);
   }
@@ -913,14 +910,14 @@ int *visible;
 
   for (i = 0; i<8; i++)
   {
-    Assign_Vector(P[i], box->bounds[0]);
-
-    if (i & 1) P[i][X] = box->bounds[1][X];
-    if (i & 2) P[i][Y] = box->bounds[1][Y];
-    if (i & 4) P[i][Z] = box->bounds[1][Z];
+    P[i][X] = (i & 1) ? box->bounds[1][X] : box->bounds[0][X];
+    P[i][Y] = (i & 2) ? box->bounds[1][Y] : box->bounds[0][Y];
+    P[i][Z] = (i & 4) ? box->bounds[1][Z] : box->bounds[0][Z];
 
     if (box->Trans != NULL)
+    {
       MTransPoint(P[i], P[i], box->Trans);
+    }
 
     transform_point(P[i]);
   }
@@ -977,9 +974,9 @@ int *visible;
   {
     Assign_Vector(P[i], hfield->bounding_box->bounds[0]);
 
-    if (i & 1) P[i][X] = hfield->bounding_box->bounds[1][X];
-    if (i & 2) P[i][Y] = hfield->bounding_box->bounds[1][Y];
-    if (i & 4) P[i][Z] = hfield->bounding_box->bounds[1][Z];
+    P[i][X] = (i & 1) ? hfield->bounding_box->bounds[1][X] : hfield->bounding_box->bounds[0][X];
+    P[i][Y] = (i & 2) ? hfield->bounding_box->bounds[1][Y] : hfield->bounding_box->bounds[0][Y];
+    P[i][Z] = (i & 4) ? hfield->bounding_box->bounds[1][Z] : hfield->bounding_box->bounds[0][Z];
 
     if (hfield->Trans != NULL)
     {
@@ -1059,6 +1056,7 @@ int *visible;
     if (New.x2 < Project->x2) Project->x2 = New.x2;
     if (New.y1 > Project->y1) Project->y1 = New.y1;
     if (New.y2 < Project->y2) Project->y2 = New.y2;
+
     *visible = TRUE;
   }
 }
@@ -1130,6 +1128,7 @@ int *visible;
     if (New.x2 < Project->x2) Project->x2 = New.x2;
     if (New.y1 > Project->y1) Project->y1 = New.y1;
     if (New.y2 < Project->y2) Project->y2 = New.y2;
+
     *visible = TRUE;
   }
 }
@@ -1419,11 +1418,9 @@ int infinite;
 
     for (i = 0; i < 8; i++)
     {
-      Assign_BBox_Vect(P[i], Object->BBox.Lower_Left);
-
-      P[i][X] += ((i & 1) ? Object->BBox.Lengths[X] : 0.0);
-      P[i][Y] += ((i & 2) ? Object->BBox.Lengths[Y] : 0.0);
-      P[i][Z] += ((i & 4) ? Object->BBox.Lengths[Z] : 0.0);
+      P[i][X] = ((i & 1) ? Object->BBox.Lengths[X] : 0.0) + Object->BBox.Lower_Left[X];
+      P[i][Y] = ((i & 2) ? Object->BBox.Lengths[Y] : 0.0) + Object->BBox.Lower_Left[Y];
+      P[i][Z] = ((i & 4) ? Object->BBox.Lengths[Z] : 0.0) + Object->BBox.Lower_Left[Z];
 
       transform_point(P[i]);
 
